const os = require("os");
const path = require("path");
const fs = require("fs-extra");
const CRCHash = require("../CRCHash.js");

module.exports=
class ArchiveBase
{
	/**
	 * constructor
	 */
	constructor()
	{
		if(new.target === ArchiveBase)
		{
			throw new TypeError();
		}
	}

	/**
	 * @param  {string} strSource
	 * @param  {string} strDestination
	 *
	 * @return {string} strSourceCRCHash
	 */
	async compressAndTest(strSource, strDestination)
	{
		const crcHash = new CRCHash();
		const strTmpPath = path.join(os.tmpdir(), "compress-" + new Date().getTime().toString());

		try {
			await this.compress(strSource, strDestination);
			await this.decompress(strDestination, strTmpPath);

			const strSourceCRCHash = await crcHash.generate(strSource);
			const strTmpCRCHash = await crcHash.generate(strTmpPath);
			if (strSourceCRCHash !== strTmpCRCHash)
			{
				throw new Error("Compressed data does not seem to match original data!");
			}

			return strSourceCRCHash;
		}
		catch (error)
		{
			console.error("Compressing failed: ");
			console.error(error);
		}
		finally
		{
			await fs.remove(strTmpPath);
		}
	}

	/**
	 * @param  {string} strSource
	 * @param  {string} strDestination
	 */
	async compress(strSource, strDestination)
	{
		throw new Error("Not implemented.");
	}

	/**
	 * @param  {string} strSource
	 * @param  {string} strDestination
	 */
	async decompress(strSource, strDestination)
	{
		throw new Error("Not implemented.");
	}
};
