const ArchiverBase = require("./ArchiverBase");
const archiver = require("archiver");
const fs = require("fs");
const path = require("path");
const yauzl = require("yauzl");
const fsExtra = require("fs-extra");
const Glob = require("glob").Glob;

module.exports=
class ZipArchiver extends ArchiverBase
{
	/**
	 * constructor
	 */
	constructor()
	{
		super();
	}

	/**
	 * @param  {string} strSourceDir
	 * @param  {string} strDestination
	 */
	async compress(strSourceDir, strDestination)
	{
		let fnResolve;
		let fnReject;

		const promise = new Promise(
			(_fnResolve, _fnReject) =>
			{
				fnResolve = _fnResolve;
				fnReject = _fnReject;
			}
		);

		new Glob("**", {dot: true, nodir: true, cwd: strSourceDir}, (error, arrMatches) =>
		{
			if (error)
			{
				fnReject(error);
				return;
			}

			try
			{
				const output = fs.createWriteStream(strDestination);
				output.on('close', () =>fnResolve());
				output.on('warning', (error) =>fnReject(error));
				output.on('error', (error) =>fnReject(error));

				const archive = archiver('zip', {
					zlib: {level: 9} // Sets the compression level.
				});
				archive.pipe(output);

				for (let strMatch of arrMatches)
				{
					archive.file(path.join(strSourceDir, strMatch), {name: strMatch});
				}

				archive.finalize();
			}
			catch (error)
			{
				fnReject(error);
			}
		});

		return promise;
	}

	/**
	 * @param  {string} strSource
	 * @param  {string} strDestination
	 */
	async decompress(strSource, strDestination)
	{
		let fnResolve;
		let fnReject;

		const promise = new Promise(
			(_fnResolve, _fnReject) => 
			{
				fnResolve = _fnResolve;
				fnReject = _fnReject;
			}
		);

		try
		{
			await fsExtra.ensureDir(strSource);
		}
		catch (error)
		{
			if (error.code !== "EEXIST")
			{
				fnReject(err);
				return;
			}
		}

		yauzl.open(strSource, {lazyEntries: true}, (err, zipfile) =>
		{
			if (err)
			{
				fnReject(err);
				return;
			}

			zipfile.readEntry();
			zipfile.on("entry", async(entry) =>
			{
				const fullPath = path.join(strDestination, this._adaptFileName(entry.fileName));
				if (/\/$/.test(entry.fileName)) // is directory
				{
					try
					{
						await fsExtra.ensureDir(fullPath);
					}
					catch (error)
					{
						if (err.code !== "EEXIST")
						{
							fnReject(err);
							return;
						}
					}

					zipfile.readEntry();
				}
				else
				{
					try
					{
						await fsExtra.ensureDir(path.dirname(fullPath));
					}
					catch (error)
					{
						if (err && err.code !== "EEXIST")
						{
							fnReject(error);
							return;
						}
					}

					zipfile.openReadStream(entry, (err, readStream) =>
					{
						if (err)
						{
							fnReject(err);
							return;
						}

						readStream.on("end", () =>
						{
							zipfile.readEntry();
						});

						readStream.pipe(fs.createWriteStream(fullPath));
					});
					}
				});

				zipfile.once("end", () =>
				{
					this.decompressFinishedCount++;
					fnResolve();
				});

				this.decompressStartedCount++;
			}
		);

		return promise;
	}


	_adaptFileName(strFileName)
	{
		return strFileName
	}
};
