const fs = require("fs");
const fsx = require("fs-extra");
const clear = require("clear");
const request = require("request");
const progress = require("request-progress");
const path = require("path");
const DownloadProgressInfo = require("./DownloadProgressInfo");

module.exports =
class DownloadTask
{

	constructor()
	{
		this._downloadProgressInfoArray = [];
	}

	/**
	 * @param  {string} strURL
	 */
	getFileNameFromUrl(strURL)
	{
		let lastSlashPosition = strURL.lastIndexOf("/");
		return strURL.slice(lastSlashPosition + 1);
	}

	/**
	 * @param  {string} strAbsoluteFilePath
	 * 
	 * @returns {boolean}
	 */
	async _existsFile(strAbsoluteFilePath)
	{
		return new Promise((resolve, reject) =>
		{
			fs.exists(strAbsoluteFilePath, (bExists) =>
			{
				resolve(bExists);
			});
		});
	}
	
	/**
	 * @param  {string} strURL
	 * @param  {string} strAbsoluteFolderPath
	 */
	async downloadItem(strURL, strAbsoluteFolderPath)
	{
		let fnResolve;
		let fnReject;
		
		const promise = new Promise((_fnResolve, _fnReject) => 
		{
			fnResolve = _fnResolve;
			fnReject = _fnReject;
		});
		
		const strFileName = this.getFileNameFromUrl(strURL);
		const strTempFileName = strFileName + ".part";
		const strAbsoluteFilePath = path.join(strAbsoluteFolderPath, strFileName);
		const strAbsoluteTempFilePath = path.join(strAbsoluteFolderPath, strTempFileName);
		const bExistsFile = await this._existsFile(strAbsoluteFilePath);
		const bExistsTempFile = await this._existsFile(strAbsoluteTempFilePath);

		if(bExistsFile && !bExistsTempFile)
		{
			fnResolve();
			return promise;
		}

		if(bExistsTempFile)
		{
			await fsx.unlink(strAbsoluteTempFilePath);
		}

		this._downloadProgressInfoArray.push(new DownloadProgressInfo(strFileName));
		
		const downloadProgress = progress(request(strURL), {});

		downloadProgress.on("progress", (state) => 
		{
			let downloadProgressInfo = this._downloadProgressInfoArray.filter((item) => 
			{
				return (item.strFileName === strFileName);
			});
			if(downloadProgressInfo.length !== 1)
			{
				fnReject(new Error("Cannot find the right download to update"));
			}
			else
			{
				downloadProgressInfo[0].currentSpeed = (state.speed / 1024).toFixed(2);
				downloadProgressInfo[0].percent = (state.percent * 100).toFixed(2);
			}
		});

		downloadProgress.on("error", (error) => 
		{
			fnReject(error);
		});

		downloadProgress.on("end", async () => 
		{
			let downloadProgressInfo = this._downloadProgressInfoArray.filter((item) => 
			{
				return (item.strFileName === strFileName);
			});

			if(downloadProgressInfo.length !== 1)
			{
				fnReject(new Error("Cannot find the right download to update"));
			}
			else
			{
				downloadProgressInfo[0].percent = 100;
				downloadProgressInfo[0].isFinished = true;
			}

			// rename the temp file to the real fileName.
			await fsx.move(strAbsoluteTempFilePath, strAbsoluteFilePath);
			fnResolve();
		});

		downloadProgress.pipe(fs.createWriteStream(strAbsoluteTempFilePath));
		return promise;
	}
	
	/**
	 * 
	 */
	async printDownloadProgress()
	{
		clear();
		this._downloadProgressInfoArray.forEach((element) => 
		{
			console.log(element.strFileName + " " + element.percent + "% " + element.currentSpeed + "KB/s");
		});
	}

	/**
	 * @returns {bool}
	 */
	isDownloadFinished()
	{
		let counterFinished = 0;
		this._downloadProgressInfoArray.forEach((element) => 
		{
			if(element.isFinished)
			{
				counterFinished++;
			}
		});
		if(counterFinished === this._downloadProgressInfoArray.length)
		{
			return true;
		}
		else
		{
			return false;
		}
	}
};
