const fs = require("fs-extra");
const path = require("path");
const TarGZArchiver = require("./Archivers/TarGZArchiver");
const ZipArchiver = require("./Archivers/ZipArchiver");

module.exports=
class Packager
{
	/**
	 * @param  {string} strDistribution
	 * @param  {string} strNodeFolder
	 * @param  {string} strFolderDestination
	 */
	async copyNodeBinary(strDistribution, strNodeFolder, strFolderDestination)
	{
		let strNodeBinaryPath;
		let strDestinationPath;
		if(strDistribution === "linux")
		{
			strNodeBinaryPath = __dirname + "/temp/" + strNodeFolder + "/bin/node";
			strDestinationPath = strFolderDestination + "/node";
		}
		else
		{
			strNodeBinaryPath = __dirname + "/temp/" + strNodeFolder + "/node.exe";
			strDestinationPath = strFolderDestination + "/node.exe";
		}

		return await fs.copy(strNodeBinaryPath, strDestinationPath);
	}

	/**
	 * @param  {string} strDistribution
	 * @param  {string} strArchiveName
	 * @param  {string} strPackageFolder
	 * @param  {string} strOutputFolder
	 *
	 * @return {string} 
	 */
	async package(strDistribution, strArchiveName, strPackageFolder, strOutputFolder)
	{
		// create archive.
		let archiver = strDistribution === "win"
			? new ZipArchiver()
			: new TarGZArchiver()
		;

		let strExtension = strDistribution === "win"
			? ".zip"
			: ".tar.gz"
		;

		let destinationPath = path.join(
			strOutputFolder,
			strDistribution,
			strArchiveName + strExtension);

		const strSourceCRCHash = await archiver.compressAndTest(strPackageFolder, destinationPath);

		return strSourceCRCHash;
	}
};
