const Logger = require("../index.js");

const fs = require("fs-promise");
const assert = require("assert");

class CustomError extends Error 
{};

module.exports =
class AllTests
{
	constructor()
	{
		this.logger = new Logger();
	}


	/**
	 * @returns {undefined}
	 */
	async runTests()
	{
		await this.testOneInsert();
		await this.testMultipleInserts();
		await this.testInsertAndUpdate();
		await this.testGetData();
		await this.testWriteDataToFile();
		await this.cleanUp();
	}


	/**
	 * @returns {undefined}
	 */
	async testOneInsert()
	{
		await this.logger.init("Test", "Test.db", "Testing");
		await this.logger.log(new Error("One insert"));
		
		let arrResult = await this.logger.getDataByDateString("2000-01-01T00:00Z", "2050-01-01T00:00Z");
		assert(arrResult.length === 1, "One insert failed");

		console.log("One insert successful");
	}


	/**
	 * @returns {undefined}
	 */
	async testMultipleInserts()
	{
		await this.logger.init("Test", "Test.db", "Testing");
		let nInserts = 5;

		for(let i = 1; i <= nInserts; i++)
		{
			await this.logger.log(new Error("Insert " + i));
		}

		let arrResult = await this.logger.getDataByDateString("2000-01-01T00:00Z", "2050-01-01T00:00Z");
		
		assert(arrResult.length === nInserts, "Multiple inserts failed");

		console.log("Multiple inserts successful");
	}


	/**
	 * @returns {undefined}
	 */
	async testInsertAndUpdate()
	{
		await this.logger.init("Test", "Test.db", "Testing");

		for(let i = 1; i <= 5; i++)
		{
			await this.logger.log(new Error("Insert and update error"));
		}

		let arrResult = await this.logger.getDataByDateString("2000-01-01T00:00Z", "2050-01-01T00:00Z");
		assert(arrResult.length === 1, "Insert and update failed");

		console.log("Insert and update successful");
	}


	/**
	 * @returns {undefined}
	 */
	async testGetData()
	{
		await this.logger.init("Test", "Test.db", "Testing");

		await this.logger.log(new Error("Default Error"));
		await this.logger.log(new CustomError("Custom Error"));

		let arrResult = await this.logger.getDataByErrorType("CustomError");
		assert(arrResult.length === 1, "getDataByErrorType failed");

		arrResult = await this.logger.getDataByErrorMessage("Custom Error");
		assert(arrResult.length === 1, "getDataByErrorMessage failed");

		console.log("Get data successful");
	}


	/**
	 * @returns {undefined}
	 */
	async testWriteDataToFile()
	{
		await this.logger.init("Test", "Test.db", "Testing");
		await this.logger.log(new Error("Testing Error"));

		await this.logger.writeDataOnDisk();

		let fileStats = await fs.stat(this.logger.databasePath);

		assert(fileStats["size"] !== 0, "writeDataToFile failed");		

		console.log("Write data successful");

	}
	

	/**
	 * @returns {undefined}
	 */
	async cleanUp()
	{
		console.log("Cleaning up");

		await fs.unlink(this.logger.databasePath);
	}
};

