"use strict";

const BaseValidatorException = require("../Base/BaseValidatorException");

module.exports =
class ISCSINameValidatorException extends BaseValidatorException
{
	constructor(strMessage, nCode = 0)
	{
		super(strMessage, nCode);
	}


	/*	RFC 3720  ; 3.2.6.3.  iSCSI Name Structure
	*
	*	 iqn.       - iSCSI Qualified name
	*	 eui.      - Remainder of the string is an IEEE EUI-64
	*	             identifier, in ASCII-encoded hexadecimal.
	*/
	static get INVALID_TYPE_DESIGNATOR() { return 1; }

	/*	RFC 3721 ; 1.1
	*/
	static get NO_IQN_MANDATORY_PART() { return 2; }

	/*	RFC 3721
	*	A date code specifying the year and month in which the
	*	organization registered the domain or sub-domain name used as the
	*	naming authority string.
	*	yyyy-mm , jan == 01, Dec == 12
	*/
	static get WRONG_DATE_FORMAT() { return 3; }

	/*
	* EUI name must only contain hexadecimal characters.
	*/
	static get NOT_HEXADECIMAL() { return 10; }

	/*
	* EUI name must be exactly 64 bits long.
	*/
	static get NOT_64_BIT() { return 11; }

	/*	RFC 3720; 3.2.6.3.1.
	*    An optional, colon (:) prefixed, string within the character
	*    set and length boundaries that the owner of the domain name
	*    deems appropriate.  This may contain product types, serial
	*    numbers, host identifiers, or software keys (e.g., it may
	*    include colons to separate organization boundaries).  With the
	*    exception of the colon prefix, the owner of the domain name can
	*    assign everything after the reversed domain name as desired.
	*    It is the responsibility of the entity that is the naming
	*    authority to ensure that the iSCSI names it assigns are
	*    worldwide unique.  For example, "Example Storage Arrays, Inc.",
	*    might own the domain name "example.com". 
	*/
	/*  Additional restrictions (not specified by RFC) :
	*	- names should be printable & non-whitespace ASCII
	*	- entire iqn. name size should be no greater than 223 characters
	*	- names issued by com.bigstep should be a valid label ( see PHorse/Validators/DomainName/ )
	*/
	static get INVALID_CHARACTER() { return 6; }
	static get NAME_TOO_LONG() { return 7; }

};
