const assert = require("assert");

class Filesystem
{
	/**
	 * Class contains filesystem properties, needed for disk plays.
	 * Block size might have a default of 1024, since it's an optional parameter for some plays.
	 * 
	 * @param {string} strFilesystemType 
	 * @param {number} nBlockSize
	 * @throws {Error}
	 */
	constructor(strFilesystemType, nBlockSize)
	{
		assert(typeof strFilesystemType === "string", "Filesystem type must be set and must be a string.");
		assert(Number.isInteger(nBlockSize) && this._checkPowerOf2(nBlockSize), "Block size must be set, must be an integer and a power of 2.");

		this._strFilesystemType = strFilesystemType;
		this._nBlockSize = nBlockSize;
	}

	/**
	 * @returns {string}
	 */
	get filesystemType()
	{
		return this._strFilesystemType;
	}

	/**
	 * @returns {number}
	 */
	get blockSize()
	{
		return this._nBlockSize;
	}

	/**
	 * Checks if given number is a power of 2.
	 * 
	 * @param {number} nNumber 
	 * @returns {boolean}
	 */
	_checkPowerOf2(nNumber)
	{
		return (Math.log(nNumber) / Math.log(2)) % 1 === 0;
	}

}

module.exports = Filesystem;
