const assert = require("assert");

class ISCSIDConfiguration
{
	/**
	 * 
	 * @param {string} striSNSAdress 
	 * @param {number} niSNSPort 
	 * @param {string} strISCSIAuthMethod 
	 * @param {string} strISCSIUsername 
	 * @param {string} strISCSIPassword 
	 * @param {number} nSessionsCount 
	 * @param {string} strStartupMode 
	 */
	constructor(striSNSAdress, niSNSPort, strISCSIAuthMethod, strISCSIUsername, strISCSIPassword, nSessionsCount, strStartupMode)
	{
		assert(typeof striSNSAdress === "string" && striSNSAdress.trim().length !== 0, "iSNS adress must be set and must be a string.");
		assert(Number.isInteger(niSNSPort) && niSNSPort > 0 && niSNSPort < 65535, "iSNS port must be set and must be a valid port number.");
		assert(typeof strISCSIAuthMethod === "string" && strISCSIAuthMethod.trim().length !== 0, "ISCSI authentication method must be set and must be a string.");
		assert(typeof strISCSIUsername === "string" && strISCSIUsername.trim().length !== 0, "ISCSI username must be set and must be a string.");
		assert(typeof strISCSIPassword === "string" && strISCSIPassword.trim().length !== 0, "ISCSI password must be set and must be a string.");
		assert(Number.isInteger(nSessionsCount), "Number of sessions must be set and must be a number.");
		assert(typeof strStartupMode === "string" && strStartupMode.trim().length !== 0, "Startup mode must be set and must be a string.");

		this._striSNSAdress = striSNSAdress;
		this._niSNSPort = niSNSPort;
		this._strISCSIAuthMethod = strISCSIAuthMethod;
		this._strISCSIUsername = strISCSIUsername;
		this._strISCSIPassword = strISCSIPassword;
		this._nSessionsCount = nSessionsCount;
		this._strStartupMode = strStartupMode;
	}

	/**
	 * @returns {string}
	 */
	get iSNSAdress()
	{
		return this._striSNSAdress;
	}

	/**
	 * @returns {number}
	 */
	get iSNSPort()
	{
		return this._niSNSPort;
	}

	/**
	 * @returns {string}
	 */
	get ISCSIAuthMethod()
	{
		return this._strISCSIAuthMethod;
	}

	/**
	 * @returns {string}
	 */
	get ISCSIUsername()
	{
		return this._strISCSIUsername;
	}

	/**
	 * @returns {string}
	 */
	get ISCSIPassword()
	{
		return this._strISCSIPassword;
	}

	/**
	 * @returns {number}
	 */
	get sessionsCount()
	{
		return this._nSessionsCount;
	}

	/**
	 * @returns {string}
	 */
	get startupMode()
	{
		return this._strStartupMode;
	}
}

module.exports = ISCSIDConfiguration;
