const net = require("net");
const assert = require("assert");
const ISCSINameValidator = require("validators").ISCSIName.ISCSINameValidator;

class ISCSIDriveCredentials
{
	/**
	 * @param {number} nDriveID
	 * @param {number} nLUNIndex 
	 * @param {string} strPortalAddress 
	 * @param {number} nPortalPort 
	 * @param {string} strTargetIQN
	 */
	constructor(nDriveID, nLUNIndex, strPortalAddress, nPortalPort, strTargetIQN)
	{
		assert(Number.isInteger(nDriveID), "Drive ID must be set and must be a number.");
		assert(Number.isInteger(nLUNIndex) && nLUNIndex < 255 && nLUNIndex > 0, "LUN index must be set and must be a valid Lun number.");
		assert(typeof strPortalAddress === "string" && net.isIPv4(strPortalAddress), "Portal IP address must be set, must be a string and must be a valid IPv4 IP.");
		assert(Number.isInteger(nPortalPort) && nPortalPort > 0 && nPortalPort < 65535, "Portal port must be set and must be a valid port number.");
		ISCSINameValidator.assertValid(strTargetIQN);

		this._nDriveID = nDriveID;
		this._nLUNIndex = nLUNIndex;
		this._strPortalAddress = strPortalAddress;
		this._nPortalPort = nPortalPort;
		this._strTargetIQN = strTargetIQN;
	}

	/**
	 * @returns {number}
	 */
	get driveID()
	{
		return this._nDriveID;
	}

	/**
	 * @returns {number}
	 */
	get LunIndex()
	{
		return this._nLUNIndex;
	}

	/**
	 * @returns {string}
	 */
	get portalAddress()
	{
		return this._strPortalAddress;
	}

	/**
	 * @returns {number}
	 */
	get portalPort()
	{
		return this._nPortalPort;
	}

	/**
	 * @returns {string}
	 */
	get targetIQN()
	{
		return this._strTargetIQN;
	}
}

module.exports = ISCSIDriveCredentials;
