const assert = require("assert");

class ModuleCallback
{
	/**
	 * @param {string} strModuleName
	 * @param {string} strModuleFunction
	 * @param {Object} arrParams
	 * @param {booolean} bPersistent
	 */
	constructor(strModuleName, strModuleFunction, arrParams, bPersistent)
	{
		arrParams = arrParams || [];
		assert(typeof strModuleName === "string", "strModuleName variable must be set!");
		assert(typeof strModuleFunction === "string", "strModuleFunction variable must be set!");

		this._strModuleName = strModuleName;
		this._strModuleFunction = strModuleFunction;
		this._arrParams = arrParams;
		this._bPersistent = bPersistent || false;
	}

	/**
	 * @returns {string}
	 */
	get moduleName()
	{
		return this._strModuleName;
	}

	/**
	 * @returns {string}
	 */
	get moduleFunction()
	{
		return this._strModuleFunction;
	}

	/**
	 * @returns {Array}
	 */
	get params()
	{
		return this._arrParams;
	}


	/**
	 * @returns {boolean}
	 */
	get persistent()
	{
		return this._bPersistent;
	}

	/**
	 * @returns {Object}
	 */
	toJSON()
	{
		return {
			moduleName: this.moduleName,
			moduleFunction: this.moduleFunction,
			params: this.params,
			persistent: this.persistent
		};
	}

	/**
	 * @param {ModuleCallback} otherModuleCallback
	 * @returns {boolean}
	 */
	equals(otherModuleCallback)
	{
		for (const nKey in this.params) 
		{
			if (this.params[nKey] !== otherModuleCallback.params[nKey]) 
			{
				return false;
			}
		}

		return this.moduleName === otherModuleCallback.moduleName
			&& this.moduleFunction === otherModuleCallback.moduleFunction
			&& this.persistent === otherModuleCallback.persistent;
	}
};

module.exports = ModuleCallback;
