const assert = require("assert");

class Partition
{
	/**
	 * Holds information for a single partition.
	 * For the moment, only index is used.
	 * 
	 * @param {number} nIndex 
	 * @param {string} strType 
	 * @param {number} nSize 
	 * @param {number} nOffset 
	 */
	constructor(nIndex, strType, nSize, nOffset)
	{
		this._strType = strType || null;
		this._nSize = nSize || null;
		this._nOffset = nOffset || null;

		assert(Number.isInteger(nIndex), "Partition index must be set and must be an integer.");

		if(this._strType !== null)
		{
			assert(typeof this._strType === "string", "If partition type is set, it must be a string.");
		} 

		if(this._nSize !== null)
		{
			assert(Number.isInteger(this._nSize), "If partition size is set, it must be an integer.");
		}

		if(this._nOffset !== null)
		{
			assert(Number.isInteger(this._nOffset), "If partition offset is set, it must be an integer.");
		}

		this._nIndex = nIndex;
	}

	/**
	 * @returns {number}
	 */
	get index()
	{
		return this._nIndex;
	}

	/**
	 * @returns {string}
	 */
	get type()
	{
		return this._strType;
	}

	/**
	 * @returns {number}
	 */
	get size()
	{
		return this._nSize;
	}

	/**
	 * @returns {number}
	 */
	get offset()
	{
		return this._nOffset;
	}
}

module.exports = Partition;
