const Partition = require("./Partition");
const assert = require("assert");

class PartitionTable
{
	/**
	 * Holds information for the partition table of a disk.
	 * 
	 * @param {string} strType
	 * @param {index} nPartitionCount 
	 * @param {Array} arrPartitions
	 * @throws {Error}
	 */
	constructor(strType, nPartitionCount, arrPartitions)
	{
		assert(typeof strType === "string", "Partition table type must be set and must be a string.");
		assert(Number.isInteger(nPartitionCount), "Partition count must be set.");
		assert(arrPartitions !== null && undefined !== arrPartitions, "Partitions array must exist.");
		
		for(let nPartitionIndex = 0; nPartitionIndex < arrPartitions.length; nPartitionIndex++)
		{
			const partition = arrPartitions[nPartitionIndex];
			assert(partition instanceof Partition, "Partition " + nPartitionIndex + " must be a Partition object.");
		}
		
		this._strType = strType;
		this._nPartitionCount = nPartitionCount;
		this._arrPartitions = arrPartitions;
	}

	/**
	 * @returns {string}
	 */
	get type()
	{
		return this._strType;
	}

	/**
	 * @returns {number}
	 */
	get partitionCount()
	{
		return this._nPartitionCount;
	}

	/**
	 * @returns {Array}
	 */
	get partitions()
	{
		return this._arrPartitions;
	}

	/**
	 * @param {number} nPartitionIndex
	 * @returns {Partition}
	 */
	getPartition(nPartitionIndex)
	{
		return this._arrPartitions[nPartitionIndex];
	}
}

module.exports = PartitionTable;
