const assert = require("assert");

class VLAN 
{
	/**
	 * @param {string} strVLANName
	 * @param {string} strIp
	 * @param {number} nPrefix
	 * @param {string} strBondName
	 * @param {string} strGateway
	 * @param {Array} arrNameServers
	 * @param {Array} arrSearchDomains
	 */
	constructor(strVLANName, strIp, nPrefix, strBondName, strGateway, arrNameServers, arrSearchDomains)
	{
		assert(typeof strVLANName === "string", "strVLANName variable must be set!");
		assert(typeof strIp === "string", "strIp variable must be set!");
		assert(Number.isInteger(nPrefix), "nPrefix variable must be set!");
		assert(typeof strBondName === "string", "strNetwork variable must be set!");

		this._strVLANName = strVLANName;
		this._strIP = strIp;
		this._nPrefix = nPrefix;
		this._strBondName = strBondName;
		this._strGateway = strGateway || null;
		this._arrNameServers = arrNameServers || [];
		this._arrSearchDomains = arrSearchDomains || [];
	}

	/**
	 * @returns {string}
	 */
	get IP()
	{
		return this._strIP;
	}

	/**
	 *
	 * @returns {string}
	 */
	get vlanName()
	{
		return this._strVLANName;
	}

	/**
	 *
	 * @returns {number}
	 */
	get prefix()
	{
		return this._nPrefix;
	}

	/**
	 /**
	 * @returns {string|null}
	 */
	get gateway()
	{
		return this._strGateway;
	}

	/**
	 * @param {string|null} strGateway
	 */
	set gateway(strGateway)
	{
		this._strGateway = strGateway;
	}

	/**
	 *
	 * @returns {string}
	 */
	get bondName()
	{
		return this._strBondName;
	}

	get nameServers()
	{
		return this._arrNameServers;
	}

	get searchDomains()
	{
		return this._arrSearchDomains;
	}

	/**
	 * @returns {string}
	 */
	toJSON()
	{
		return {
			vlanName: this.vlanName,
			IP: this.IP,
			prefix: this.prefix,
			bondName: this.bondName,
			nameServers: this.nameServers,
			searchDomains: this.searchDomains
		};
	}

	/**
	 * @param {Object} objJson
	 * @returns {State}
	 */
	static createFromJSON(objJson)
	{
		const vlan = new VLAN(
			objJson.vlanName,
			objJson.IP,
			objJson.prefix,
			objJson.bondName,
			objJson.gateway,
			objJson.nameServers,
			objJson.searchDomains
		);

		return vlan;
	}
}

module.exports = VLAN;
