const ModuleResponse = require("./ModuleResponse");
const MountManagerProxySingleton = require("../../MountManager").MountManagerProxySingleton;

class MountManager
{
	/**
	 * @param {string} strMountPoint
	 * @param {string} strDiskPath
	 * @param {string} strFilesystemType
	 * @returns {ModuleResponse}
	 */
	async mount(strMountPoint, strDiskPath, strFilesystemType)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		const strMountServiceName = await mountManager.mount(strMountPoint, strDiskPath, strFilesystemType);

		return new ModuleResponse(true, strMountServiceName);
	}

	/**
	 * @param {string} strMountPoint
	 * @param {string} strDiskPath
	 * @param {string} strFilesystemType
	 * @returns {ModuleResponse}
	 */
	async isMounted(strMountPoint, strDiskPath, strFilesystemType)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		const bIsMounted = await mountManager.isMounted(strMountPoint, strDiskPath, strFilesystemType);

		return new ModuleResponse(true, bIsMounted.toString());
	}

	/**
	 * @param {string} strMountPoint
	 * @returns {ModuleResponse}
	 */
	async unmount(strMountPoint)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		await mountManager.unmount(strMountPoint);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strMountPoint
	 * @param {string} strDiskPath
	 * @param {string} strFilesystemType
	 * @returns {ModuleResponse}
	 */
	async mountAndSetAutomount(strMountPoint, strDiskPath, strFilesystemType)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		await mountManager.mountAndSetAutomount(strMountPoint, strDiskPath, strFilesystemType);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strMountPoint
	 * @param {string} strDiskPath
	 * @param {string} strFilesystemType
	 * @returns {ModuleResponse}
	 */
	async isAutomountSet(strMountPoint, strDiskPath, strFilesystemType)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		const bIsAutomountSet = await mountManager.isAutomountSet(strMountPoint, strDiskPath, strFilesystemType);

		return new ModuleResponse(true, bIsAutomountSet.toString());
	}

	/**
	 * @param {string} strMountPoint
	 * @returns {ModuleResponse}
	 */
	async unmountAndUnsetAutomount(strMountPoint)
	{
		const mountManager = await MountManagerProxySingleton.getUsable();
		await mountManager.unmountAndUnsetAutomount(strMountPoint);

		return new ModuleResponse(true);
	}
}

module.exports = MountManager;
