const ModuleResponse = require("./ModuleResponse");
const os = require("os");
const PackageManagerProxySingleton = require("../../PackageManager").PackageManagerProxySingleton;

class PackageManager
{
	/**
	 * @param {string} strPackageName
	 *
	 * @returns {ModuleResponse}
	 */
	async install(strPackageName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const packageManager = await PackageManagerProxySingleton.getUsable();
		const bIsChanged = await packageManager.install(strPackageName);

		return new ModuleResponse(bIsChanged);
	}

	/**
	 * @param {string} strPackageName
	 *
	 * @returns {ModuleResponse}
	 */
	async remove(strPackageName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}
		
		const packageManager = await PackageManagerProxySingleton.getUsable();
		const bIsChanged = await packageManager.remove(strPackageName);

		return new ModuleResponse(bIsChanged);
	}
}

module.exports = PackageManager;
