const ModuleResponse = require("./ModuleResponse");
const os = require("os");
const ServiceManagerProxySingleton = require("../../SystemServiceManager").SystemServiceManagerProxySingleton;

class ServiceManager
{
	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async start(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await this._systemServiceManagerProxy.getUsable();

		if (await systemServiceManager.isRunning(strServiceName))
		{
			return new ModuleResponse(false);
		}

		await systemServiceManager.start(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async stop(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();

		if (!await systemServiceManager.isRunning(strServiceName))
		{
			return new ModuleResponse(false);
		}

		await systemServiceManager.stop(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async enable(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();

		if (await systemServiceManager.isEnabled(strServiceName))
		{
			return new ModuleResponse(false);
		}

		await systemServiceManager.enable(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async disable(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();

		if (!await systemServiceManager.isEnabled(strServiceName))
		{
			return new ModuleResponse(false);
		}

		await systemServiceManager.disable(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async restart(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();
		
		await systemServiceManager.restart(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {ModuleResponse}
	 */
	async reload(strServiceName)
	{
		if (os.platform() !== "linux")
		{
			throw new Error("Not implemented. Only linux modules avaialble at this point.");
		}

		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();
		await systemServiceManager.reload(strServiceName);

		return new ModuleResponse(true);
	}

	/**
	 * @returns {ModuleResponse}
	 */
	async reloadDaemon()
	{
		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();
		await systemServiceManager.reloadDaemon();

		return new ModuleResponse(true);
	}

	/**
	 * @param {string} strServiceName 
	 * @returns {ModuleResponse}
	 */
	async isRunning(strServiceName)
	{
		const systemServiceManager = await ServiceManagerProxySingleton.getUsable();
		const bServiceIsRunning = await systemServiceManager.isRunning(strServiceName);

		return new ModuleResponse(true, bServiceIsRunning.toString());
	}

	/**
	 * @returns {ModuleResponse}
	 */
	async selfStop()
	{
		return await this.stop("BSIAgent");
	}
}
;

module.exports = ServiceManager;
