const FileModuleSingleton = require("../Singletons/FileSingleton");

class BaseTemplate
{
	/**
	 * Render the output by interpolating params
	 *
	 * @param {Object} objVariables
	 */
	async getRenderedTemplate(objVariables)
	{
		throw new Error("the getRenderedTemplate method must be overriden");
	}

	/**
	 * @param {Object} objVariables
	 */
	validate(objVariables)
	{
		throw new Error("The validate method must be overriden");
	}

	/**
	 * Validate params and return the rendered the output by interpolating params
	 *
	 * @param {Object} objVariables
	 */
	async render(objVariables)
	{
		this.validate(objVariables);

		return (await this.getRenderedTemplate(objVariables)).trim();
	}

	/**
	 * Render the output by interpolating params and write it to the file
	 *
	 * @param {string} strPath
	 * @param {Object} objVariables
	 * @param {Object} objSettings
	 *
	 * @returns {ModuleResponse}
	 */
	async renderToFile(strPath, objVariables, objSettings)
	{
		objVariables = objVariables || {};
		objSettings = objSettings || {};

		let strContent = await this.render(objVariables);

		return await FileModuleSingleton.save(strContent, strPath, objSettings.userID, objSettings.groupID, objSettings.mode);
	}
}

module.exports = BaseTemplate;
