const assert = require("assert");
const VLAN = require("../../Models/VLAN");
const BaseTemplate = require("./BaseTemplate");

class IfcfgVLANTemplate extends BaseTemplate
{
	/**
	 * @param {VLAN} vlan
	 * @returns {string}
	 */
	getRenderedTemplate(vlan)
	{
		return `
DEVICE=${vlan.bondName}.${vlan.vlanName}
BOOTPROTO=none
ONBOOT=yes
IPADDR=${vlan.IP}
PREFIX=${vlan.prefix}
VLAN=yes
PEERDNS=yes
${vlan.gateway ? `GATEWAY=${vlan.gateway}` : ""}
${vlan.searchDomains.length ? `SEARCH=${vlan.searchDomains.join(",")}` : ""}
${vlan.nameServers.map((strNameServer, nKey) => `DNS${nKey}=${strNameServer}`).join("\n")}
`;
	}

	/**
	 * @param {VLAN} vlan
	 */
	validate(vlan)
	{
		assert(vlan instanceof VLAN);
	}
}

module.exports = IfcfgVLANTemplate;
