const BasePlay = require("./BasePlay");
const FileSingleton = require("../Modules").FileSingleton;
const assert = require("assert");

class EnablePersistentLoggingPlay extends BasePlay
{
	/**
	 * Constructor
	 */
	constructor()
	{
		super();

		this._strConfigPath = "/etc/systemd/journald.conf";
		this.addStep(this.changeConfigFile.bind(this));
	}

	/**
	 * @throws {Error}
	 */
	async validateParams()
	{
		assert(this._objParams.enabled !== null && undefined !== this._objParams.enabled, "This play must have params set!");
	}

	/**
	 * @returns {boolean}
	 */
	async changeConfigFile()
	{
		const bJournaldPresent = await FileSingleton.exists(this._strConfigPath);
		if (!bJournaldPresent)
		{
			return false;
		}

		let strStorageOption = "volatile";
		if (this._objParams.enabled)
		{
			strStorageOption = "persistent";
		}

		let bIsChanged = false;

		let changeFileResponse = await FileSingleton.addLine("Storage=" + strStorageOption, this._strConfigPath, "Storage=");
		bIsChanged = bIsChanged || changeFileResponse.isChanged();

		changeFileResponse = await FileSingleton.addLine("SystemMaxUse=1G", this._strConfigPath, "SystemMaxUse=");
		bIsChanged = bIsChanged || changeFileResponse.isChanged();

		return bIsChanged;
	}
}

module.exports = EnablePersistentLoggingPlay;
