const VLAN = require("../Models").VLAN;
const OSInfo = require("../../Facts").Models.OSInfo;
const ModuleCallback = require("../Models").ModuleCallback;

class State
{
	constructor()
	{
		this.VLANs = [];
		this._isBondConfigured = false;
		this._rebootRequestedDate = null;
		this._arrStartupCallbacks = [];
		this._OSInfo = {};
	}

	/**
	 * @returns {void}
	 */
	clearRebootRequest()
	{
		this._rebootRequestedDate = null;
	}

	/**
	 * @returns {Object}
	 */
	toJSON()
	{
		const arrVLANs = [];

		for (const vlan of this.VLANs)
		{
			arrVLANs.push(vlan.toJSON());
		}

		return {
			VLANs: arrVLANs,
			isBondConfigured: this.isBondConfigured,
			rebootRequestedDate: this.rebootRequestedDate ? this.rebootRequestedDate.getTime() : null,
			startupCallbacks: this.startupCallbacks,
			OSInfo: this.OSInfo
		};
	}

	/**
	 * @param {Object} objJson
	 * @returns {State}
	 */
	static createFromJSON(objJson)
	{
		const state = new State();

		state.isBondConfigured = objJson.isBondConfigured;
		state.rebootRequestedDate = objJson.rebootRequestedDate ? new Date(objJson.rebootRequestedDate) : null;
		for (const objCallback of objJson.startupCallbacks)
		{
			state.addStartupCallback(new ModuleCallback(
				objCallback.moduleName,
				objCallback.moduleFunction,
				objCallback.params,
				objCallback.persistent || false)
			);
		}
		state.OSInfo = OSInfo.createFromJSON(objJson.OSInfo);

		for (const objVLAN of objJson.VLANs)
		{
			state.VLANs.push(VLAN.createFromJSON(objVLAN));
		}

		return state;
	}

	/**
	 * @param {number} nIndex
	 */
	removeStartupCallback(nIndex)
	{
		this.startupCallbacks.splice(nIndex);
	}

	/**
	 * @param {ModuleCallback} callback
	 */
	addStartupCallback(callback)
	{
		for (const objCallback of this.startupCallbacks)
		{
			if (objCallback.equals(callback))
			{
				return;
			}
		}

		this.startupCallbacks.push(callback);
	}

	/**
	 * @returns {Array}
	 */
	get startupCallbacks()
	{
		return this._arrStartupCallbacks;
	}

	/**
	 * @param {Array} arrStartupCallbacks
	 */
	set startupCallbacks(arrStartupCallbacks)
	{
		this._arrStartupCallbacks = arrStartupCallbacks;
	}

	/**
	 * @returns {Date}
	 */
	get rebootRequestedDate()
	{
		return this._rebootRequestedDate;
	}

	/**
	 * @param {Date} rebootRequestedDate
	 */
	set rebootRequestedDate(rebootRequestedDate)
	{
		this._rebootRequestedDate = rebootRequestedDate;
	}

	/**
	 * @returns {Array}
	 */
	get VLANs()
	{
		return this._arrVLANs;
	}

	/**
	 * @param {Array} arrVLANs
	 */
	set VLANs(arrVLANs)
	{
		this._arrVLANs = arrVLANs;
	}

	/**
	 * @returns {boolean}
	 */
	get isBondConfigured()
	{
		return this._isBondConfigured;
	}

	/**
	 * @param {boolean} bIsBondConfigured
	 */
	set isBondConfigured(bIsBondConfigured)
	{
		this._isBondConfigured = bIsBondConfigured;
	}

	/**
	 * @returns {OSInfo}
	 */
	get OSInfo()
	{
		return this._OSInfo;
	}

	/**
	 * @param {OSInfo} osInfo
	 */
	set OSInfo(osInfo)
	{
		this._OSInfo = osInfo;
	}

	/**
	 * @param {VLAN} vlan
	 * @returns {boolean}
	 */
	addVLAN(vlan)
	{
		for (const existstingVLAN of this._arrVLANs)
		{
			if (existstingVLAN.name === vlan.name)
			{
				return false;
			}
		}

		this._arrVLANs.push(vlan);

		return true;
	}

	/**
	 * @param {string} strVLANName
	 * @returns {boolean}
	 */
	removeVLAN(strVLANName)
	{
		for (const existstingVLANKey in this._arrVLANs)
		{
			if (this._arrVLANs[existstingVLANKey].name === strVLANName)
			{
				this._arrVLANs.splice(existstingVLANKey, 1);
				return true;
			}
		}

		return false;
	}
}

module.exports = State;
