const State = require("./State");
const path = require("path");
const atomicWriteFileSystemSingleton = require("../../FileSystem").AtomicWriteFileSystemSingleton;

class StateHandler
{
	constructor()
	{
		this._strSavePath = path.join("/var", "lib", "MetalCloudAgent", "state.json");
	}

	/**
	 * @param {State} state
	 */
	async save(state)
	{
		const strSerializedState = JSON.stringify(state, null, "\t");

		await atomicWriteFileSystemSingleton.writeFile(
			this._strSavePath, strSerializedState, {overwrite: true}
		);
	}

	/**
	 * return {State}
	 */
	async get()
	{
		try
		{
			const strContent = await atomicWriteFileSystemSingleton.readFile(this._strSavePath);
			if (strContent.trim() === "")
			{
				return new State();
			}
			return State.createFromJSON(JSON.parse(strContent));
		}
		catch (error)
		{
			if (error.code === "ENOENT")
			{ // path does not exist
				return new State();
			}

			throw error;
		}
	}
}

module.exports = StateHandler;
