// @ts-check

const ChildProcess = require("child-process-promise");
const Utils = require("jskippy-node").Utils;

class CommandExecutor
{
	/**
	 * @param {string} strCommand 
	 * @param {Object} [objConfig]
	 * @param {number} [objConfig.nExpectedExitCode = 0]
	 * @param {boolean} [objConfig.bIgnoreParse = true]
	 * @returns {Promise<string>}
	 * @throws {Error}
	 */
	static async execCommandProcess(strCommand, {nExpectedExitCode = 0, bIgnoreParse = true} = {})
	{
		const objOptions = {
			capture: [ "stdout", "stderr" ],
			shell: true
		};

		const commandProcessPromise = ChildProcess.spawn(strCommand, [], objOptions);
		const commandProcess = commandProcessPromise.childProcess;

		commandProcess.stdout.pipe(process.stdout);
		commandProcess.stderr.pipe(process.stderr);

		let strCommandOutput = "";

		try
		{
			let objResult = await commandProcessPromise;
			strCommandOutput = objResult.stdout.toString();

			if(objResult.code !== nExpectedExitCode)
			{
				throw new Error(`The expected exit code: ${nExpectedExitCode} does not match with the produced one: ${objResult.code}`);
			}

			// Check result for correct JSON format
			if(!bIgnoreParse)
			{
				Utils.safeJSONParse(objResult);
			}
		}
		catch(error)
		{
			console.error(`Error trying to execute command "${strCommand}" \n${error.stack}`);
		}

		return strCommandOutput;
	}
}

module.exports = CommandExecutor;
