// @ts-check

const CommandBase = require("./CommandBase");

class DiskDetection extends CommandBase
{
	constructor(objConfig = {})
	{
		super(objConfig);
	}

	/**
	 * @returns {string}
	 */
	static get command()
	{
		return "lsblk -b -S -P -o KNAME,TYPE,VENDOR,TRAN,MODEL,SERIAL,SIZE,ROTA | grep \"TYPE=\\\"disk\\\"\"";
	}

	/**
	 * Parses the raw output from the command, must me runned after execCommand
	 * 
	 * @param {string} [strResult=null]
	 * @returns {Promise<Object>} _objParsedCommandOutput
	 */
	async _parseCommandOutput(strResult = null)
	{
		if(strResult)
		{
			this._strRawCommandOutput = strResult;
		}

		if(!this._strRawCommandOutput || this._strRawCommandOutput.trim() === "")
		{
			throw new Error(`Empty output string for command ${this.constructor.name}.`);
		}

		const objColumnMapping = {
			"MODEL": "server_disk_model",
			"SIZE": "server_disk_size_gb",
			"SERIAL": "server_disk_serial",
			"VENDOR": "server_disk_vendor",
			"ROTA": "server_disk_type",
			"KNAME": "server_disk_label"
		};

		const arrDisks = [];	
		const arrDiskStrings = this._strRawCommandOutput.split("\n");

		arrDiskStrings.forEach((strDiskString, nIndex) => 
		{
			const objDisk = {};
			const arrDiskPropertyRaw = strDiskString.split("\" ");

			arrDiskPropertyRaw.forEach((strDiskPropertyRaw, nIndex) => 
			{
				const arrDiskProperty = strDiskPropertyRaw.split("=\"");
				
				if(objColumnMapping.hasOwnProperty(arrDiskProperty[0]))
				{
					objDisk[objColumnMapping[arrDiskProperty[0]]] = arrDiskProperty[1].replace("\"", "").trim();
				}
			});

			let bValidDisk = false;

			Object.values(objColumnMapping).forEach((strDiskInfo, nIndex) => 
			{
				if(Object.keys(objDisk).includes(strDiskInfo))
				{
					bValidDisk = true;
				}
			});

			if(bValidDisk)
			{
				arrDisks.push(objDisk);
			}
		});
		
		arrDisks.forEach((objServerDisk, nIndex) => 
		{
			objServerDisk.server_disk_size_gb = Math.round(objServerDisk.server_disk_size_gb / 1024 / 1024 / 1024);
			objServerDisk.server_disk_type = objServerDisk.server_disk_type === "1" ? DiskDetection.DiskTypes.DISK_TYPE_HDD : DiskDetection.DiskTypes.DISK_TYPE_SSD;
		});

		this._objParsedCommandOutput.disks = arrDisks;
		return this._objParsedCommandOutput;
	}

	get parsedOutput()
	{
		return this._objParsedCommandOutput;
	}

	/**
	 * @returns {string}
	 */
	static get osPlatform()
	{
		return "linux";
	}

	async test()
	{
		
	}

	static get DiskTypes()
	{
		return {
			"DISK_TYPE_HDD": "HDD",
			"DISK_TYPE_SSD": "SSD"
		};
	}
}

module.exports = new DiskDetection({bMustBeFiltered: false, bMustBeParsed: true});
