// @ts-check

const CommandBase = require("./CommandBase");

class EFIbootmgr extends CommandBase
{
	constructor(objConfig = {})
	{
		super(objConfig);
	}

	static get command()
	{
		return "efibootmgr -v 2>&1";
	}

	/**
	 * Parses the raw output from the command, must me runned after execCommand
	 * 
	 * @param {string} [strResult=null]
	 * @returns {Promise<Object>} _objParsedCommandOutput
	 */
	async _parseCommandOutput(strResult = null)
	{
		if(strResult)
		{
			this._strRawCommandOutput = strResult;
		}

		if(!this._strRawCommandOutput || this._strRawCommandOutput.trim() === "")
		{
			throw new Error(`Empty output string for command ${this.constructor.name}.`);
		}

		const bEnabledEFIVariables = !this._strRawCommandOutput.includes("EFI variables are not supported on this system.");

		this._objParsedCommandOutput = {
			enabled_efi_variables: bEnabledEFIVariables
		};
		
		return this._objParsedCommandOutput;
	}

	get parsedOutput()
	{
		return this._objParsedCommandOutput;
	}

	static get osPlatform()
	{
		return "linux";
	}

	async test()
	{
		
	}
}

module.exports = new EFIbootmgr({bMustBeFiltered: false, bMustBeParsed: true});
