// @ts-check

const CommandBase = require("./CommandBase");

class Lscpu extends CommandBase
{
	constructor(objConfig = {})
	{
		super(objConfig);
	}

	static get command()
	{
		return "lscpu";
	}

	/**
	 * Parses the raw output from the command, must me runned after execCommand
	 * 
	 * @param {string} [strResult=null]
	 * @returns {Promise<Object>} _objParsedCommandOutput
	 */
	async _parseCommandOutput(strResult = null)
	{
		if(strResult)
		{
			this._strRawCommandOutput = strResult;
		}

		if(!this._strRawCommandOutput || this._strRawCommandOutput.trim() === "")
		{
			throw new Error(`Empty output string for command ${this.constructor.name}.`);
		}

		const objCPUInfoNameMap = {
			"Architecture": "server_processor_arhitecture",
			"Thread(s) per core": "server_processor_threads",
			"CPU(s)": "server_processor_count"
		};

		const objCPUInfo = {};
		const arrInfoRows = this._strRawCommandOutput.split("\n");

		arrInfoRows.forEach((strInfoRow, nIndex) => 
		{
			const arrCPUInfoSplitedInfo = strInfoRow.split(":");

			if(arrCPUInfoSplitedInfo.length === 2)
			{
				let strCPUPropertyName = arrCPUInfoSplitedInfo[0];
				const strCPUPropertyValue = arrCPUInfoSplitedInfo[1].trim();
	
				strCPUPropertyName = objCPUInfoNameMap.hasOwnProperty(strCPUPropertyName) 
					? objCPUInfoNameMap[strCPUPropertyName] 
					: strCPUPropertyName;
	
				objCPUInfo[strCPUPropertyName] = strCPUPropertyValue;
			}
		});
		
		this._objParsedCommandOutput = objCPUInfo;
		return this._objParsedCommandOutput;
	}

	static get osPlatform()
	{
		return "linux";
	}

	get parsedOutput()
	{
		return this._objParsedCommandOutput;
	}

	async test()
	{
		
	}

	static get DISK_TYPE_NVME()
	{
		return "NVME";
	}
}

module.exports = new Lscpu({bMustBeFiltered: false, bMustBeParsed: true});
