// @ts-check

const CommandBase = require("./CommandBase");

class NVMeDiskDetection extends CommandBase
{
	constructor(objConfig = {})
	{
		super(objConfig);
	}

	static get command()
	{
		return "lsblk -b -P -o KNAME,TYPE,VENDOR,TRAN,MODEL,SERIAL,SIZE,ROTA | grep \"TYPE=\\\"disk\\\"\" | grep nvme";
	}

	/**
	 * Parses the raw output from the command, must me runned after execCommand
	 * 
	 * @param {string} [strResult=null]
	 * @returns {Promise<Object>} _objParsedCommandOutput
	 */
	async _parseCommandOutput(strResult = null)
	{
		if(strResult)
		{
			this._strRawCommandOutput = strResult;
		}

		if(!this._strRawCommandOutput || this._strRawCommandOutput.trim() === "")
		{
			throw new Error(`Empty output string for command ${this.constructor.name}.`);
		}
		this._objParsedCommandOutput.disks = [];

		const arrNVMeDiskInfo = this._strRawCommandOutput.split("\n");
		arrNVMeDiskInfo.forEach((strDiskInfo, nIndex) =>
		{
			try
			{
				const objDisk = {};
				const arrDiskPropertyRaw = strDiskInfo.split("\" ");
	
				arrDiskPropertyRaw.forEach((strDiskPropertyRaw, nIndex) => 
				{
					const arrDiskProperty = strDiskPropertyRaw.split("=\"");
					objDisk[arrDiskProperty[0]] = arrDiskProperty[1];
				});
	
				
				this._objParsedCommandOutput[objDisk["KNAME"]] = {
					size: objDisk["SIZE"],
					vendor: objDisk["VENDOR"]
				}
				
				// const nServerDiskSize = parseInt(objDisk["SIZE"], 10);

				// if(isNaN(nServerDiskSize))
				// {
				// 	throw new Error(`Invalid server_disk_size generated from parsing: ${strDiskInfo}`);
				// }
	
				// this._objParsedCommandOutput.disks.push({
				// 	server_disk_size_gb: Math.round(nServerDiskSize) / 1024 / 1024 / 1024,
				// 	server_disk_vendor: objDisk["VENDOR"],
				// 	server_disk_label: objDisk["KNAME"],
				// 	server_disk_type: NVMeDiskDetection.DISK_TYPE_NVME
				// });
			}
			catch(error)
			{
				console.error(error.stack);
			}
		});

		return this._objParsedCommandOutput;
	}

	get parsedOutput()
	{
		return this._objParsedCommandOutput;
	}

	static get osPlatform()
	{
		return "linux";
	}

	/**
	 * @returns {string} "SSD"
	 */
	static get DISK_TYPE_SSD()
	{
		return "SSD";
	}

	static get DISK_TYPE_NVME()
	{
		return "NVME";
	}

	async test()
	{
		
	}
}

module.exports = new NVMeDiskDetection({bMustBeFiltered: false, bMustBeParsed: true});
