
const os = require("os");
const Commands = require("./Commands");

class UtilityOSInfo
{
	constructor()
	{

	}

	/**
	 * @returns {Object} objResults
	 * 
	 * Run the batch of available commands and returns the parsed and filtered output grouped by command name
	 */
	static async runCommandBatch()
	{
		const fnHandledPromise = async(promise) => 
		{
			try
			{
				await promise;
			}
			catch(error)
			{
				console.error(error);
			}
		};

		const strOSPlatform = os.platform();

		const arrErrors = [];
		const objResults = {};

		let bRunned = false;
		
		try
		{
			const arrPromises = [];

			if(strOSPlatform === "win32")
			{
				bRunned = true;
			}
			
			if(!bRunned && strOSPlatform === "linux")
			{
				arrPromises.push(fnHandledPromise(Commands.Lshw.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.HBA.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.CCMcfg.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.NVMe.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.NVMeDiskDetection.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.DiskDetection.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.EFIbootmgr.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.NetworkInterfaces.execCommand(/*objConfig*/)));

				arrPromises.push(fnHandledPromise(Commands.Lscpu.execCommand(/*objConfig*/)));
				
				// Must wait for all commands to finish to centralize the results
				await Promise.all(arrPromises);

				// Check if all commands produced output
				for(let strCommandName in Commands)
				{
					if(Commands[strCommandName].output)
					{
						objResults[strCommandName] = Commands[strCommandName].output;
					}
					else
					{
						arrErrors.push(strCommandName);
					}
				}

				if(arrErrors.length !== 0)
				{
					console.error(`Empty result on commands: [ ${arrErrors.join(", ")} ]`);
				}
			}
		}
		catch(error)
		{
			console.error(error.stack);
		}

		return objResults;
	}

	/**
	 * List of available shell commands
	 * 
	 * @returns {Object} Commands
	 */
	static get Commands()
	{
		return Commands;
	}
}


module.exports = UtilityOSInfo;
