const argv = require("minimist")(process.argv.slice(2));
const os = require("os");
//const path = require("path");

module.exports = 
class Bootstrap
{
	constructor()
	{
		throw new Error("Static usage only.");
	}

	static validateCLIArguments()
	{
		const arrAllowedCLIKeys = ["service-install", "service-run", "cli-run", "mac-address-spoof", "ip-address-spoof", "uuid-spoof", "mode-bdk", "mode-agent"];
		const arrOptionalKeys = ["mac-address-spoof", "ip-address-spoof", "uuid-spoof", "mode-bdk", "mode-agent"];
		const arrCLIKeys = [ ...new Set(Object.keys(argv).concat(arrOptionalKeys)) ];
		
		const arrFoundKeys = [];
		const arrBadKeys = [];

		for(let strKey of arrCLIKeys)
		{
			if(arrAllowedCLIKeys.includes(strKey))
			{
				arrFoundKeys.push(strKey);
			}
			else
			{
				if(strKey === "_")
				{
					arrBadKeys.concat(Object.keys(argv["_"]));
				}
				else
				{
					arrBadKeys.push(strKey);
				}
			}
		}

		if(arrBadKeys.length)
		{
			console.log(`Unknown CLI argument(s) ${arrBadKeys.join(", ")}.`);
			console.log(os.EOL);
		}

		if(arrBadKeys.length || !arrFoundKeys.length)
		{
			console.log(Bootstrap.usageHelp());

			process.exit(1);
		}

		return argv;
	}


	static usageHelp()
	{
		const strCLIUsageHelp = (`
			Usage:
				node src/main.js --cli-run
				node src/main.js --cli-run --mode-bdk

			Run modes (mutually exclusive):
				--cli-run
				--service-run
				--service-install

			Optional service modes (mutually exclusive):
				--mode-bdk
				--mode-deploy (default)
			
			Optional development flags:
				--mac-address-spoof
				--ip-address-spoof
				--uuid-spoof

			Deploy agent development usage: node src/main.js --cli-run --mac-address-spoof=00:0c:29:01:01:03 --ip-address-spoof=192.168.137.100
			BDK development usage: node src/main.js --cli-run --mode-bdk --uuid-spoof=98124D56-4786-8BAC-A970-A33C46CB5EC3
		`).replace(/^\t{3,3}/mg, "").trim();

		return strCLIUsageHelp;
	}
};
