class OSInfo
{
	/**
	 * @param {string} strPlatform
	 * @param {string} strDistribution
	 * @param {string} strRelease
	 * @param {string} strArchitecture
	 */
	constructor(strPlatform, strDistribution, strRelease, strArchitecture)
	{
		this._strPlatform = strPlatform;
		this._strDistribution = strDistribution;
		this._strRelease = strRelease;
		this._strArchitecture = strArchitecture;
	}

	/**
	 * @param {Object} objJSON
	 * @returns {State}
	 */
	static createFromJSON(objJSON)
	{
		if (objJSON === null || objJSON === undefined) 
		{
			return new OSInfo();
		}
		
		return new OSInfo(
			objJSON.platform,
			objJSON.distribution,
			objJSON.release,
			objJSON.architecture
		);
	}

	/**
	 * @returns {Object}
	 */
	toJSON()
	{
		return {
			platform: this.platform,
			distribution: this.distribution,
			release: this.release,
			architecture: this.architecture
		};
	}

	/**
	 * @returns {string}
	 */
	get platform()
	{
		return this._strPlatform;
	}

	/**
	 * @returns {string}
	 */
	get distribution()
	{
		return this._strDistribution;
	}

	/**
	 * @returns {string}
	 */
	get release()
	{
		return this._strRelease;
	}

	/**
	 * @returns {string}
	 */
	get architecture()
	{
		return this._strArchitecture;
	}

	/**
	 * @param {string} strRelease
	 */
	set release(strRelease)
	{
		this._strRelease = strRelease;
	}

	/**
	 * @param {string} strDistribution
	 */
	set distribution(strDistribution)
	{
		this._strDistribution = strDistribution;
	}

	/**
	 * @param {*} osInfo
	 * @returns {boolean}
	 */
	isEqual(osInfo)
	{
		return this.platform === osInfo.platform
			&& this.distribution === osInfo.distribution
			&& this.release === osInfo.release
			&& this.architecture === osInfo.architecture;
	}
}

module.exports = OSInfo;
