const AtomicWriteFileSystemSingleton = require("../FileSystem/").AtomicWriteFileSystemSingleton;
const LoggedChildProcessSingleton = require("../LoggedChildProcessSingleton");

class BasePackageManager
{
	/**
	 * @returns {string}
	 */
	getBinaryName()
	{
		throw new Error("Must be implemented");
	}

	/**
	 * @returns {boolean}
	 */
	async isUsable()
	{
		throw new Error("Must be implemented");
	}

	/**
	 * @param {string} strPackageName
	 * @returns {boolean}
	 */
	async install(strPackageName)
	{
		throw new Error("Must be implemented");
	}

	/**
	 * @param {string} strPackageName
	 * @returns {boolean}
	 */
	async remove(strPackageName)
	{
		throw new Error("Must be implemented");
	}

	/**
	 * @param {string} strName
	 * @param {Array} arrPaths
	 * @returns {string}
	 */
	async findFullPath(strName, arrPaths)
	{
		for (const strPath of arrPaths)
		{
			const strFullPath = `${strPath}/${strName}`;
			if (await AtomicWriteFileSystemSingleton.exists(strFullPath))
			{
				return strFullPath;
			}
		}

		return null;
	}

	/**
	 * @param {string} strCommand
	 *
	 * @returns {string}
	 */
	async execute(strCommand)
	{
		let fnResolve;
		let fnReject;
		let strStdout = "";
		let strStderr = "";

		const childProcess = await LoggedChildProcessSingleton.spawn(strCommand, {shell: true});
		const promise = new Promise((fnPromiseResolve, fnPromiseReject) =>
		{
			fnResolve = fnPromiseResolve;
			fnReject = fnPromiseReject;
		});

		childProcess.stdout.on("data", (data) =>
		{
			strStdout += data;
		});
		childProcess.stderr.on("data", (data) =>
		{
			strStderr += data;
		});

		childProcess.on("error", (error) =>
		{
			fnReject(new Error(`Could not execute command: ${strCommand}`, error));
		});

		childProcess.on("exit", (nResultCode, strSignal) =>
		{
			strStdout = strStdout.trim();
			fnResolve(strStdout);
		});

		return promise;
	}
}
;

module.exports = BasePackageManager;
