const BasePackageManager = require("./BasePackageManager");

class YUM extends BasePackageManager
{
	/**
	 * @returns {string}
	 */
	getBinaryName()
	{
		return "yum";
	}

	/**
	 * @returns {boolean}
	 */
	async isUsable()
	{
		return await this.getFullBinaryPath() !== null;
	}

	/**
	 * @returns {string|null}
	 */
	async getFullBinaryPath()
	{
		const arrBinPaths = ["/sbin", "/usr/sbin", "/bin", "/usr/bin"];

		return await this.findFullPath(this.getBinaryName(), arrBinPaths);
	}

	/**
	 * @param {string} strPackageName
	 * @returns {boolean}
	 */
	async install(strPackageName)
	{
		const strStdout = await this.execute(`${await this.getFullBinaryPath()} install -y ${strPackageName}`);

		return strStdout.indexOf("Nothing to do") === -1;
	}

	/**
	 * @param {string} strPackageName
	 * @returns {boolean}
	 */
	async remove(strPackageName)
	{
		const strStdout = await this.execute(`${await this.getFullBinaryPath()} remove -y ${strPackageName}`);

		return strStdout.indexOf("No Packages marked for removal") === -1;
	}
}

module.exports = YUM;
