const BaseSystemServiceManager = require("./BaseSystemServiceManager");
const loggedChildProcessSingleton = require("../LoggedChildProcessSingleton");

class GenericSystemServiceManager extends BaseSystemServiceManager
{
	/**
	 * @returns {string}
	 */
	getBinaryName()
	{
		return "service";
	}

	/**
	 * @returns {boolean}
	 */
	async isUsable()
	{
		return await this.getFullBinaryPath() !== null;
	}

	/**
	 * @returns {string|null}
	 */
	async getFullBinaryPath()
	{
		const arrBinPaths = ["/sbin", "/usr/sbin", "/bin", "/usr/bin"];

		return await this.findFullPath(this.getBinaryName(), arrBinPaths);
	}

	/**
	 * @param {string} strServiceName
	 */
	async start(strServiceName)
	{
		await loggedChildProcessSingleton.spawn(`${await this.getFullBinaryPath()} ${strServiceName} start`, {shell: true});
	}

	/**
	 * @param {string} strServiceName
	 */
	async stop(strServiceName)
	{
		await loggedChildProcessSingleton.spawn(`${await this.getFullBinaryPath()} ${strServiceName} stop`, {shell: true});
	}

	/**
	 * @param {string} strServiceName
	 */
	async enable(strServiceName)
	{
		throw new Error("Can't enable service with the generic service manager \"service\". Install systemd!");
	}

	/**
	 * @param {string} strServiceName
	 */
	async disable(strServiceName)
	{
		throw new Error("Can't disable service with the generic service manager \"service\". Install systemd!");
	}

	/**
	 * @param {string} strServiceName
	 */
	async reload(strServiceName)
	{
		if (!await this.isRunning(strServiceName))
		{
			throw new Error("Can not reload a service that is not running!");
		}

		await loggedChildProcessSingleton.spawn(`${await this.getFullBinaryPath()} ${strServiceName} reload`, {shell: true});
	}

	/**
	 * @param {string} strServiceName
	 */
	async restart(strServiceName)
	{
		await loggedChildProcessSingleton.spawn(`${await this.getFullBinaryPath()} ${strServiceName} restart`, {shell: true});
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {boolean}
	 */
	async isRunning(strServiceName)
	{
		const strCommand = `${await this.getFullBinaryPath()} ${strServiceName} status`;
		const strStdoutResponse = await this.executeWithRetry(strCommand, (strStdout) => strStdout.indexOf("Active: activating") !== -1);

		return strStdoutResponse.indexOf("Active: active") !== -1;
	}

	/**
	 * @param {string} strServiceName
	 *
	 * @returns {boolean}
	 */
	async isEnabled(strServiceName)
	{
		throw new Error("Can't check if service is enabled with the generic service manager \"service\". Install systemd!");
	}
};

module.exports = GenericSystemServiceManager;
