const genericSystemServiceManagerSingleton = require("./GenericSystemServiceManagerSingleton");
const systemdSingleton = require("./SystemdSingleton");

class SystemServiceManagerProxy
{
	constructor()
	{
		this._packageManagers = [genericSystemServiceManagerSingleton, systemdSingleton];
		this._usableServiceManager = null;
	}

	/**
	 * @returns {BaseSystem}
	 */
	async getUsable()
	{
		if (this._usableServiceManager !== null)
		{
			return this._usableServiceManager;
		}

		for (const serviceManager of this._packageManagers)
		{
			if (await serviceManager.isUsable())
			{
				this._usableServiceManager = serviceManager;
				return this._usableServiceManager;
			}
		}

		throw new Error("No system service manager found on this machine!");
	}
}

module.exports = SystemServiceManagerProxy;
