"use strict";

// @TODO: When publishing, manually copy the node binary from here https://nodejs.org/en/download/current/ to this project root.
// Relying on having the node binary distributed with this application.
// Of course, that is not necessary when developing, if having the right global node version.
// @TODO: use this to send HW info to BSI. https://github.com/scravy/node-macaddress

const CLIBootstrap = require("./CLI/Bootstrap");
const AgentIdentity = require("./Agent/Identity");
const AgentService = require("./Agent/Service");
const DeployAgentUpdater = require("./Agent/Updater");

const BSIWebSocketClientManager = require("./Agent/BSIWebSocketClientManager");
const BSIDeployAgentEndpoint = require("./Agent/Endpoints/BSIDeployAgent");
const BDKAgentEndpoint = require("./Agent/Endpoints/BDKAgent");
const ModuleCallbackRunner = require("./Agent/Modules/ModuleCallbackRunner");
const LoggedChildProcess = require("./LoggedChildProcess");
const stateHandler = require("./Agent/State").StateHandlerSingleton;
const factsProvider = require("./Facts/").FactsProviderSingleton;

const JSkippy = require("jskippy-node");
JSkippy.ConsoleFormat.setupGlobalConsole();

//const Logger = require("structuredlogger");
const assert = require("assert");
//const logger = new Logger();

// Avoiding "DeprecationWarning: Unhandled promise rejections are deprecated. In the future, promise rejections that are not handled will terminate the Node.js process with a non-zero exit code."
// by actually doing the respective exit with non-zero code.
// This allows the watcher to restart this process.
process.on(
	"unhandledRejection", 
	async (reason, promise) => 
	{
		try
		{
			//await logger.logErrorPersistent(reason);
		}
		catch(error)
		{
			console.log("Logger error: " + error.stack);
		}
		finally
		{
			console.log("Unhandled Rejection at: Promise", promise, "reason", reason);
			process.exit(1);
		}
	}
);

process.on(
	"uncaughtException",
	(error) => {
		console.log("[" + process.pid + "] Unhandled exception.");
		console.error(error);

		process.exit(1);
	}
);

(async () => 
{
	// Exits process if required params are not there.
	const argv = CLIBootstrap.validateCLIArguments();
	const bInstallService = !!argv["service-install"];
	const bRunAsCLI = !!argv["cli-run"];
	const bRunAsService = !!argv["service-run"];
	const bBDKMode = !!argv["mode-bdk"];
	let bDeployMode = !!argv["mode-deploy"];

	if(!bBDKMode && !bDeployMode)
	{
		bDeployMode = true;
	}

	if(bBDKMode + bDeployMode !== 1)
	{
		console.error("mode-bdk and mode-deploy are mutually exclusive. Choose only one, or none to default to deploy agent mode.");
		process.exit(1);
	}

	let endpoint;
	let strAgentMode;
	let deployAgentUpdater;

	// Development shortcuts.
	strAgentMode = bBDKMode ? "bdk" : "deploy";
	const strMACAddressSpoof = argv["mac-address-spoof"] ? argv["mac-address-spoof"] : null;
	const strIPAddressSpoof = argv["ip-address-spoof"] ? argv["ip-address-spoof"] : null;
	const strUUIDSpoof = argv["uuid-spoof"] ? argv["uuid-spoof"] : null;
	const agentIdentity = new AgentIdentity(strAgentMode, strMACAddressSpoof, strIPAddressSpoof, strUUIDSpoof);
	await agentIdentity.init();

	if(bBDKMode)
	{
		endpoint = new BDKAgentEndpoint(agentIdentity);
	}
	else
	{
		assert(bDeployMode);
		deployAgentUpdater = new DeployAgentUpdater(agentIdentity, new LoggedChildProcess());
		endpoint = new BSIDeployAgentEndpoint(deployAgentUpdater, agentIdentity);
	}

	if(!bRunAsCLI && !bRunAsService)
	{
		console.log(CLIBootstrap.usageHelp());
		process.exit(0);
	}
	
	// Initialize logger before other opereations
	//await logger.init(AgentIdentity.applicationName, "Error.db", "Errors");
	const bsiWebSocketClientManager = new BSIWebSocketClientManager(agentIdentity, endpoint, factsProvider);

	const agent = new AgentService(
		bRunAsService,
		bsiWebSocketClientManager,
		stateHandler,
		new ModuleCallbackRunner(),
		factsProvider
	);
	
	if(bInstallService)
	{
		throw new Error("Not implemented.");
	}

	assert(bRunAsService || bRunAsCLI);
	await agent.run();
})();
