const exec = require("child_process").exec;
const os = require("os");
const path = require("path");
const nexe = require("nexe");

const strPlatform = os.platform();
const strArchitecture = os.arch();


const JSkippy = require("jskippy-node");
JSkippy.ConsoleFormat.setupGlobalConsole();


// Avoiding "DeprecationWarning: Unhandled promise rejections are deprecated. In the future, promise rejections that are not handled will terminate the Node.js process with a non-zero exit code."
// by actually doing the respective exit with non-zero code.
// This allows the watcher to restart this process.
process.on(
	"unhandledRejection", 
	async(reason, promise) => 
	{
		console.log("Unhandled Rejection at: Promise", promise, "reason", reason);
		process.exit(1);
	}
);


async function runCLICommand(strCommand)
{
	const processCommand = exec(strCommand, {maxBuffer: 500 * 1024 * 1024});
	processCommand.stdout.pipe(process.stdout);
	processCommand.stderr.pipe(process.stderr);
	return new Promise(async(fnResolve, fnReject) => {
		processCommand.on("error", fnReject);
		processCommand.on("exit", (nCode) => {
			if(nCode === 0)
			{
				fnResolve();
			}
			else
			{
				fnReject(new Error("Failed with error code " + nCode));
			}
		});
	});
}


(async() => {
	process.chdir(path.dirname(__dirname));

	console.log("Building.");
	await runCLICommand(path.resolve("./node_modules/.bin/webpack"));
	
	await new Promise((fnResolve, fnReject) => {
		nexe.compile(
			{
				input: path.resolve(path.dirname(__dirname), "dist/nexe/bsiagent.min.js"),
				output: path.resolve(path.dirname(__dirname), "dist/nexe/" + strPlatform + "/" + strArchitecture + "/bsiagent" + (strPlatform === "win32" ? ".exe" : "")),
				nodeVersion: "7.9.0", // node version
				nodeTempDir: path.resolve(path.dirname(__dirname), "tmp"),
				nodeConfigureArgs: ["opt", "val"], // for all your configure arg needs.
				nodeMakeArgs: ["-j", "4"], // when you want to control the make process.
				nodeVCBuildArgs: ["nosign", strArchitecture], // when you want to control the make process for windows.
				// By default "nosign" option will be specified
				// You can check all available options and its default values here:
				// https://github.com/nodejs/node/blob/master/vcbuild.bat
				// python: "path/to/python", // for non-standard python setups. Or python 3.x forced ones.
				// resourceFiles: [ "path/to/a/file" ], // array of files to embed.
				// resourceRoot: [ "path/" ], // where to embed the resourceFiles.
				flags: true, // use this for applications that need command line flags.
				jsFlags: "--use_strict", // v8 flags
				//startupSnapshot: "path/to/snapshot.js", // when you want to specify a script to be
				// added to V8"s startup snapshot. This V8
				// feature deserializes a heap to save startup time.
				// More information in this blog post:
				// http://v8project.blogspot.de/2015/09/custom-startup-snapshots.html
				framework: "node" // node, nodejs, or iojs
			}, 
			(error, result) => {
				if(error)
				{
					fnReject(error);
				}
				else
				{
					fnResolve(result);
				}
			}
		);
	});

	console.log("Done.");
})();

